# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: preferen.tcl,v 1.4 1999/05/09 19:54:38 jfontain Exp $}


namespace eval preferences {

    variable rcFileName ~/.moodssrc
    # list of: label, initialize procedure, edit procedure, check procedure, apply procedure, list of children
    variable hierarchy {
        {canvas canvas::initialize canvas::edit canvas::check canvas::apply canvas::saveList {}}
        {viewers viewers::initialize viewers::edit viewers::check viewers::apply viewers::saveList {}}
    }
    variable container
    variable interface

    proc load {} {
        variable rcFileName

        if {![file readable $rcFileName]} return
        set file [::open $rcFileName] ;### detect errors and report failures to user ###
        foreach {name value} [read $file] {
            set $name $value
        }
        close $file
    }

    proc edit {} {
        variable hierarchy
        variable container
        variable interface

        set dialog [new dialogBox .\
            -buttons oc -default o -title {moodss: Preferences} -x [winfo pointerx .] -y [winfo pointery .]\
            -command preferences::done\
        ]
        set frame [frame $widget::($dialog,path).frame]

        set tree [blt::hierbox $frame.tree\
            -font $font::(mediumBold) -selectmode single -selectbackground lightgray -hideroot 1 -borderwidth 1\
            -highlightthickness 0 -takefocus 0 -width 150\
        ]
        set container [frame $frame.container -borderwidth 1 -relief sunken]

        set message [message $container.message\
            -width [winfo screenwidth .] -font $font::(mediumNormal) -justify left\
            -text {\
This is the preferences dialog box for
application-wide settings.

On UNIX systems, preferences data is saved
in each user home directory under the rc
file named .moodssrc.

On Windows, data is saved in C:\.moodssrc
(by default, but depends on the HOME variable).

After selection of the category from the tree
on the left, a related dialog interface
replaces this message.

Contextual help is provided for each interface,
which may or may not allow immediately applying
new data values.

Clicking on the OK button results in the
preferences data to be written to the rc file.
}\
        ]
        pack $message -fill both -expand 1                                                   ;# initially display help message above

        bindtags $tree [list $tree [winfo toplevel $tree] all]                                              ;# ignore class bindtags
        $tree bind all <Double-ButtonPress-1> {}                                                           ;# and selection bindings
        $tree bind all <Shift-ButtonPress-1> {}
        $tree bind all <Control-ButtonPress-1> {}
        $tree bind all <B1-Motion> {}
        # only keep opening binding (no longer on double click)
### check before toggling ###
        $tree bind all <ButtonRelease-1> "$tree toggle current; $tree toggle current"

        catch {unset interface(current)}                                                               ;# currently opened interface

        foreach entry $hierarchy {
            foreach {label initialize edit check apply saveList children} $entry {
                $initialize
                set index [$tree insert end $label]                                     ;# used generated tree index as unique index
                set interface($index,edit) $edit
                set interface($index,check) $check
                set interface($index,apply) $apply
                set interface($index,saveList) $saveList
### eventually use close command for checking
                $tree entry configure $index -opencommand "preferences::open $index"
            }
        }

        pack $tree -side left -fill y -padx 2
        pack $container -fill both -expand 1 -padx 2

        dialogBox::display $dialog $frame

        wm geometry $widget::($dialog,path) 500x400                                                        ;# maintain constant size
    }

    proc open {index} {
        variable container
        variable interface

        # check validity before moving to next interface
        if {[info exists interface(current)]&&![$interface($interface(current),check)]} return

        destroyChildren $container
        $interface($index,edit) $container
        set interface(current) $index
    }

    proc done {} {
        variable interface

        foreach name [array names interface *,apply] {
            $interface($name)                                                                                    ;# apply new values
        }
        save
    }

    proc save {} {                                                                                                ;# save to rc file
        variable rcFileName

        set file [::open $rcFileName w] ;### detect errors and report failures to user ###
        puts -nonewline $file [saveList]
        close $file
    }

    proc saveList {} {
        variable interface

        set list {}
        foreach name [array names interface *,saveList] {
            set list [concat $list [$interface($name)]]
        }
        return $list
    }


    namespace eval canvas {

        proc initialize {} {
            variable height $global::canvasHeight width $global::canvasWidth
        }

        proc edit {parentPath} {
            variable height
            variable width

            set message [message $parentPath.message\
                -width [winfo screenwidth .] -font $font::(mediumNormal) -justify center\
                -text {Enter main canvas size (in pixels):}\
            ]
            grid $message -sticky nsew -row 0 -column 0 -columnspan 100                                        ;# occupy whole width
            grid rowconfigure $parentPath 0 -weight 1

            grid columnconfigure $parentPath 0 -weight 1

            set widthEntry [new spinEntry $parentPath -font $font::(mediumBold) -width 4 -list {640 800 1024 1280 1600}]
            set path $composite::($widthEntry,entry,path)
            $path configure -textvariable preferences::canvas::width
            # filter on positive integers and limit entry length
            wcb::callback $path before insert wcb::checkStrForNum {wcb::checkEntryLen 4}
            spinEntry::set $widthEntry $width
            grid [label $parentPath.width -font $font::(mediumBold) -text width:] -row 1 -column 1 -padx 2
            grid $widget::($widthEntry,path) -row 1 -column 2

            grid columnconfigure $parentPath 3 -weight 1

            set heightEntry [new spinEntry $parentPath -font $font::(mediumBold) -width 4 -list {400 480 600 768 1024 1280}]
            set path $composite::($heightEntry,entry,path)
            $path configure -textvariable preferences::canvas::height
            # filter on positive integers and limit entry length
            wcb::callback $path before insert wcb::checkStrForNum {wcb::checkEntryLen 4}
            spinEntry::set $heightEntry $height
            grid [label $parentPath.height -font $font::(mediumBold) -text height:] -row 1 -column 4 -padx 2
            grid $widget::($heightEntry,path) -row 1 -column 5

            grid [button $parentPath.apply -text Apply -command preferences::canvas::apply] -row 1 -column 6
            grid columnconfigure $parentPath 6 -weight 2

            set message [message $parentPath.help\
                -width [winfo screenwidth .] -font $font::(mediumNormal) -justify center\
                -text {\
This is the main canvas (the data viewers background)
size. The size is immediately updated when clicking
on the Apply button.}
            ]
            grid $message -sticky nsew -row 2 -column 0 -columnspan 100
            grid rowconfigure $parentPath 2 -weight 1

            bind $message <Destroy> "delete $widthEntry $heightEntry"                       ;# delete inner objects upon destruction
        }

        proc check {} {
            return 1
        }

        proc apply {} {
            variable height
            variable width

            if {![check]} return
            if {$height!=$global::canvasHeight} {            ;# update only when necessary in order not to activate trace needlessly
                set global::canvasHeight $height                                                          ;# use entry current value
            }
            if {$width!=$global::canvasWidth} {
                set global::canvasWidth $width
            }
        }

        proc saveList {} {
            return [list global::canvasHeight $global::canvasHeight global::canvasWidth $global::canvasWidth]
        }

    }


    namespace eval viewers {

        proc initialize {} {
            variable numberOfSamples $global::graphNumberOfIntervals
        }

        proc edit {parentPath} {
            variable numberOfSamples

            set message [message $parentPath.message\
                -width [winfo screenwidth .] -font $font::(mediumNormal) -justify center\
                -text {Enter number of samples for data graphs:}\
            ]
            grid $message -sticky nsew -row 0 -column 0 -columnspan 100
            grid rowconfigure $parentPath 0 -weight 1

            grid columnconfigure $parentPath 0 -weight 1

            set entry [new spinEntry $parentPath -font $font::(mediumBold) -width 4 -list {20 50 100 150 200 300 500 1000}]
            set path $composite::($entry,entry,path)
            $path configure -textvariable preferences::viewers::numberOfSamples
            # filter on positive integers and limit entry length
            wcb::callback $path before insert wcb::checkStrForNum {wcb::checkEntryLen 4}
            spinEntry::set $entry $numberOfSamples
            grid [label $parentPath.width -font $font::(mediumBold) -text samples:] -row 1 -column 1 -padx 2
            grid $widget::($entry,path) -row 1 -column 2

            grid columnconfigure $parentPath 3 -weight 1

            set message [message $parentPath.help\
                -width [winfo screenwidth .] -font $font::(mediumNormal) -justify center\
                -text {\
This is the number of samples (on the X axis)
for data graph viewers. The specified value will
not be used for existing graphs but for newly
created ones.
}
            ]
            grid $message -sticky nsew -row 2 -column 0 -columnspan 100
            grid rowconfigure $parentPath 2 -weight 1

            bind $message <Destroy> "delete $entry"                                         ;# delete inner objects upon destruction
        }

        proc check {} {
            return 1
        }

        proc apply {} {
            variable numberOfSamples

            if {![check]} return
            set global::graphNumberOfIntervals $numberOfSamples                                           ;# use entry current value
        }

        proc saveList {} {
            return [list global::graphNumberOfIntervals $global::graphNumberOfIntervals]
        }

    }

}
